# This is a part of the external applets for Cairo-Dock
# Copyright : (C) 2010-2011 by Nochka85, Fabounet and Matttbe
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL

####################
### dependancies ###
####################
import os.path
import subprocess
from CDApplet import CDApplet

####################
### Applet class ###
####################
class CDBashApplet(CDApplet):
	def __init__(self):
		# call high-level init
		self.app_folder = os.path.abspath(".")
		CDApplet.__init__(self)
	
	##### private methods #####
	
	def call(self,action):
		subprocess.call("cd " + self.app_folder + " && ./" + self.cAppletName + ".sh " + self.cAppletName + " " + self.cBusPath + " " + self.cConfFile + " " + self.cParentAppName + " " + action, shell=True)
	
	##### applet definition #####
	
	def get_config(self,keyfile):
		self.call("get_config")
	
	def end(self):
		self.call("end")
	
	def begin(self):
		self.call("begin")
	
	def reload(self):
		self.call("reload")
	
	##### callbacks #####
	
	def on_click(self,iState):
		self.call("on_click "+str(iState))
	
	def on_middle_click(self):
		self.call("on_middle_click")
		
	def on_build_menu(self):
		self.call("on_build_menu")
		
	def on_menu_select(self,iNumEntry):
		self.call("on_menu_select "+str(iNumEntry))
	
	def on_scroll(self,bScrollUp):
		self.call("on_scroll "+str(bScrollUp))
	
	def on_drop_data(self,cReceivedData):
		self.call("on_drop_data '"+cReceivedData+"'")
	
	def on_answer_dialog(self, button, answer):
		self.call("on_answer_dialog "+str(button)+" '"+str(answer)+"'")
	
	def on_shortkey(self,key):
		self.call("on_shortkey '"+key+"'")
	
	def on_change_focus(self,bIsActive):
		self.call("on_change_focus '"+str(bIsActive)+"'")
	
	def on_click_sub_icon(self, iState, cIconID):
		self.call("on_click_sub_icon '"+str(iState)+"' '"+cIconID+"'")
	



#!/usr/bin/env python3
#
# This is a part of the Cairo-Dock plug-ins.
# Copyright : (C) 2011 by Fabrice Rey
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL
#
# Developped as a part of Cairo-Dock, but usable as a stand-alone systray daemon.
# The code follows the same logic as the KDE watcher, to ensure a complete compatibility.

import sys

from gi.repository import GLib as glib
from gi.repository import GObject as gobject
g_bMainLoopInGObject = False

import dbus, dbus.service
from dbus.mainloop.glib import DBusGMainLoop

class SNWatcher(dbus.service.Object):
	bus_name_str = 'org.kde.StatusNotifierWatcher'
	bus_obj_str  = '/StatusNotifierWatcher'
	bus_iface_str = 'org.kde.StatusNotifierWatcher'
	items_list = []  # array of service+path
	hosts_list = []  # array of services
	
	def _emit_host_registered (self):
		self.StatusNotifierHostRegistered()
		return False
	
	def __init__(self):
		DBusGMainLoop(set_as_default=True)
		try: 
			self.bus = dbus.SessionBus()
			bus_name = dbus.service.BusName (self.bus_name_str, self.bus)
			print("[Cairo-Dock] Status-Notifier: registered a watcher:",bus_name)
			dbus.service.Object.__init__(self, bus_name, self.bus_obj_str)
		except dbus.DBusException:
			print('Could not open dbus. Uncaught exception.')
			return
		
		bus_object = self.bus.get_object(dbus.BUS_DAEMON_NAME, dbus.BUS_DAEMON_PATH)
		self.main = dbus.Interface(bus_object, dbus.BUS_DAEMON_IFACE)
		self.main.connect_to_signal("NameOwnerChanged", self.on_name_owner_changed)
		
		if g_bMainLoopInGObject:
			self.loop = gobject.MainLoop()
		else:
			self.loop = glib.MainLoop()
		self.loop.run()
	
	def on_name_owner_changed(self, service, prev_owner, new_owner):
		# print("name_owner_changed: %s; %s; %s" % (service, prev_owner, new_owner))
		if (new_owner == '' and not service.startswith(':')):  # a service has disappear from the bus, check if it was an item or a host.
			# search amongst the items.
			match = service+'/'
			# print("  search for ",match)
			to_be_removed=[]
			for it in self.items_list:
				# print("  check for ",it)
				if (it.startswith(match)):  # it[0:len(match)] == match
					# print("    match!")
					to_be_removed.append(it)
			for it in to_be_removed:
				self.items_list.remove (it)
				self.StatusNotifierItemUnregistered(it)

			# search amongst the hosts.
			to_be_removed=[]
			for it in self.hosts_list:
				if (it == service):
					to_be_removed.append(it)
			for it in to_be_removed:
				self.hosts_list.remove (it)
				self.StatusNotifierHostUnregistered()
		elif (service == 'com.Skype.API'):  # this stupid proprietary software only creates its item when the host appears !
			glib.timeout_add_seconds(2, self._emit_host_registered)
		
	### methods ###
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = 's', out_signature = None)
	def RegisterStatusNotifierHost(self, service):
		if (self.hosts_list.count (service) == 0):  # if not already listed
			self.hosts_list.append (service)
			self.StatusNotifierHostRegistered()
		# print('hosts:',self.hosts_list)
		sys.stdout.flush()
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = 's', out_signature = None, sender_keyword='sender')
	def RegisterStatusNotifierItem(self, serviceOrPath, sender=None):
		# build the item id: service + path
		if (serviceOrPath[0] == '/'):
			service = sender
			path = serviceOrPath
		else:
			service = serviceOrPath
			path = "/StatusNotifierItem"
		
		itemId = service + path
		# keep track of this new item, and emit the 'new' signal.
		if (self.items_list.count (itemId) == 0):  # if not already listed
			self.items_list.append (itemId)
			self.StatusNotifierItemRegistered (itemId)
	
	### Properties ###
	
	@dbus.service.method(dbus_interface = dbus.PROPERTIES_IFACE, in_signature = 'ss', out_signature = 'v')
	def Get(self, interface, property):
		if interface == 'org.kde.StatusNotifierWatcher':
			if property == 'RegisteredStatusNotifierItems':
				# print("items: ",self.items_list)
				if (len (self.items_list) != 0):
					return self.items_list
				else:  # too bad! dbus-python can't encode the GValue if 'items_list' is None or [].
					return ['']  # so we return a empty string; hopefuly the host will skip this invalid value.
			elif property == 'IsStatusNotifierHostRegistered':
				return (len (self.hosts_list) != 0)
			elif property == 'HasStatusNotifierHostRegistered':  # deprecated
				return (len (self.hosts_list) != 0)
			elif property == 'ProtocolVersion':
				return 0
	
	### Signals ###
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature='s')
	def StatusNotifierItemRegistered(self, service):
		pass
		# print("%s registered" % (service))
		# sys.stdout.flush()
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature='s')
	def StatusNotifierItemUnregistered(self, service):
		pass
		# print("%s unregistered" % (service))
		# sys.stdout.flush()
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature=None)
	def StatusNotifierHostRegistered(self):
		pass
		# print("a host has been registered")
		# sys.stdout.flush()
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature=None)
	def StatusNotifierHostUnregistered(self):
		pass
		# print("a host has been unregistered")
		# sys.stdout.flush()
	
	
if __name__ == '__main__':
	SNWatcher()



#!/usr/bin/env python3
#
# This is a part of the Cairo-Dock plug-ins.
# Copyright : (C) 2011 by Fabrice Rey
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL
#
# Developped as a part of Cairo-Dock, but usable as a stand-alone systray daemon.
# The code follows the same logic as the KDE watcher, to ensure a complete compatibility.

import sys

from gi.repository import GLib as glib
from gi.repository import GObject as gobject
g_bMainLoopInGObject = False

import dbus, dbus.service
from dbus.mainloop.glib import DBusGMainLoop

class Registrar(dbus.service.Object):
	bus_name_str = 'com.canonical.AppMenu.Registrar'
	bus_obj_str  = '/com/canonical/AppMenu/Registrar'
	bus_iface_str = 'com.canonical.AppMenu.Registrar'
	windows = {}  # table of couple (X id, menu object path)
	
	def __init__(self):
		DBusGMainLoop(set_as_default=True)
		try: 
			self.bus = dbus.SessionBus()
			bus_name = dbus.service.BusName (self.bus_name_str, self.bus)
			print("registered a registrar:",bus_name)
			dbus.service.Object.__init__(self, bus_name, self.bus_obj_str)
		except dbus.DBusException:
			print('Could not open dbus. Uncaught exception.')
			return
		
		bus_object = self.bus.get_object(dbus.BUS_DAEMON_NAME, dbus.BUS_DAEMON_PATH)
		self.main = dbus.Interface(bus_object, dbus.BUS_DAEMON_IFACE)
		#self.main.connect_to_signal("NameOwnerChanged", self.on_name_owner_changed)
		
		if g_bMainLoopInGObject:
			self.loop = gobject.MainLoop()
		else:
			self.loop = glib.MainLoop()
		self.loop.run()
	
	### methods ###
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = 'uo', out_signature = None, sender_keyword='sender')
	def RegisterWindow(self, Xid, menu, sender=None):
		self.windows[Xid] = (sender, menu)
		
		self.WindowRegistered (Xid, sender, menu)
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = 'u', out_signature = None)
	def UnregisterWindow(self, Xid):
		del self.windows[Xid]
		
		self.WindowUnregistered (Xid)
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = 'u', out_signature = 'so')
	def GetMenuForWindow(self, Xid):
		print("GetMenuForWindow(%d)..." % Xid)
		try:
			service,menu = self.windows[Xid]
			return service,menu  # dbus.ObjectPath(menu)
		except:
			return "","/"  # return an empty string to avoid ugly warnings; clients should be able to deal with it.
	
	@dbus.service.method(dbus_interface = bus_iface_str, in_signature = None, out_signature = 'a(uso)')
	def GetMenus(self):
		menus=[]
		for xid in self.windows:
			service,menu = self.windows[xid]
			menus.append((xid, service, menu))
	
	### Signals ###
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature='uso')
	def WindowRegistered(self, Xid, service, menu):
		print("%d registered" % (Xid))
		sys.stdout.flush()
	
	@dbus.service.signal(dbus_interface=bus_name_str, signature='u')
	def WindowUnregistered(self, Xid):
		print("%d unregistered" % (Xid))
		sys.stdout.flush()
	

if __name__ == '__main__':
	Registrar()



#!/usr/bin/env python3
#
# This is a part of the Cairo-Dock plug-ins.
# Copyright : (C) 2012 by Fabrice Rey
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL
#

import sys

from gi.repository import GLib as glib
from gi.repository import GObject as gobject
g_bMainLoopInGObject = False

import re
from os import popen, getpid
import dbus, dbus.service
from dbus.mainloop.glib import DBusGMainLoop
from CairoDock import CairoDock

class Launcher:
	has_count = False
	count = 0
	count_visible = False
	has_progress = False
	progress = 0.
	progress_visible = False
	has_emblem = False
	emblem = 0
	emblem_visible = False
	menu_path = ''
	urgent = False
	
	def __init__(self, app_uri=None, dock=None, dbus_name=None):
		## initialize the properties of the launcher
		self.app_uri = app_uri
		self.dock = dock
		self.dbus_name = dbus_name
		desktop_file = re.sub ('application://', '', app_uri)
		app_name = re.sub ('.desktop', '', desktop_file)
		self.id = 'class='+app_name
		
		## watch for the deconnection of the distant app (we don't want to let an icon demanding attention or with an invalid menu).
		bus = dbus.SessionBus()
		bus.watch_name_owner(self.dbus_name, self.on_name_owner_changed)
	
	def update_dbus_name(self, dbus_name):
		if dbus_name != self.dbus_name:
			self.dbus_name = dbus_name
			bus = dbus.SessionBus()
			bus.watch_name_owner(self.dbus_name, self.on_name_owner_changed)
	
	def on_name_owner_changed(self,connection_name):
		print("Launcher-API-Daemon: launcher bus name changed:",connection_name)
		if len(connection_name) == 0:  # no more distant app -> reset the launcher and its icon.
			print("Launcher-API-Daemon: -> the launcher "+ str (self.dbus_name) +" is no longer connected to its app.")
			if self.has_count and self.count_visible:
				self.dock.iface.SetQuickInfo('', self.id)
				self.has_count = False
				self.count_visible = False
			if self.has_progress and self.progress_visible:
				self.dock.iface.SetProgress (-1, self.id)
				self.has_progress = False
				self.progress_visible = False
			if self.has_emblem and self.emblem_visible:
				self.dock.iface.SetEmblem('', CairoDock.EMBLEM_TOP_RIGHT, self.id)
				self.has_emblem = False
				self.emblem_visible = False
			if self.urgent:
				self.dock.iface.DemandsAttention(False, 'default', self.id)
				self.urgent = False
			if self.menu_path != '':
				self.dock.iface.SetMenu(self.dbus_name, '', self.id)
				self.menu_path = ''
			self.dbus_name = None
			
	def set_count(self, x):
		print('Launcher-API-Daemon: set_count (%d)'%x)
		self.count = x
		self.has_count = True
		if self.count_visible:
			self.dock.iface.SetQuickInfo(str(self.count), self.id)
	
	def set_count_visible(self, x):
		print('Launcher-API-Daemon: set_count_visible (%d)'%x)
		self.count_visible = x
		if self.has_count:
			if x:
				self.dock.iface.SetQuickInfo(str(self.count), self.id)
			else:
				self.dock.iface.SetQuickInfo('', self.id)
	
	def set_progress(self, x):
		print('Launcher-API-Daemon: set_progress (%.2f)'%x)
		self.progress = x
		self.has_progress = True
		self.dock.iface.SetProgress (x, self.id)
	
	def set_progress_visible(self, x):
		print('Launcher-API-Daemon: set_progress_visible (%d)'%x)
		self.progress_visible = x
		if not x:
			self.dock.iface.SetProgress (-1, self.id)
		else:
			self.dock.iface.SetProgress (self.progress, self.id)
	
	def set_urgent(self, x):
		print('Launcher-API-Daemon: set_urgent (%d)'%x)
		self.urgent = x
		self.dock.iface.DemandsAttention(x, 'default', self.id)
	
	def set_menu(self, menu_path):
		print('Launcher-API-Daemon: set_menu (%s)'%menu_path)
		self.menu_path = menu_path
		self.dock.iface.SetMenu(self.dbus_name, menu_path, self.id)
	
	def set_emblem(self, x):
		print('Launcher-API-Daemon: set_emblem (%s)'%x)
		self.emblem = x
		self.has_emblem = True
		if self.emblem_visible:
			self.dock.iface.SetEmblem(self.emblem, CairoDock.EMBLEM_TOP_RIGHT, self.id)
	
	def set_emblem_visible(self, x):
		print('Launcher-API-Daemon: set_emblem_visible (%d)'%x)
		self.emblem_visible = x
		if self.has_emblem:
			if x:
				self.dock.iface.SetEmblem(self.emblem, CairoDock.EMBLEM_TOP_RIGHT, self.id)
			else:
				self.dock.iface.SetEmblem('', CairoDock.EMBLEM_TOP_RIGHT, self.id)
	

class ULWatcher:
	bus_iface_str = 'com.canonical.Unity.LauncherEntry'  # bus name to watch
	bus_name_str = 'com.canonical.Unity'  # bus name to claim, to signal our presence on the bus.
	launchers = {}  # list of launchers that have been seen on the bus at least once.
	
	def __init__(self):
		DBusGMainLoop(set_as_default=True)
		
		try: 
			self.bus = dbus.SessionBus()
		except Exception as exception:
			print('Launcher-API-Daemon: Could not open dbus. Uncaught exception.')
			return
		
		bus_name = dbus.service.BusName (self.bus_name_str, self.bus, allow_replacement=True)  # allow Unity shell to take ownership of the bus name; we only want to register on the bus so that other applications know they can send message on the LauncherEntry interface
		print("Launcher-API-Daemon: registered as Unity:",bus_name)
		
		self.bus.add_signal_receiver (self.on_launcher_entry_signal, dbus_interface=self.bus_iface_str, member_keyword='member', sender_keyword='sender')
		
		self.dock = CairoDock()
		
		if g_bMainLoopInGObject:
			self.loop = gobject.MainLoop()
		else:
			self.loop = glib.MainLoop()
		self.loop.run()
	
	def on_launcher_entry_signal(self, val1=None, val2=None, member=None, sender=None):  # application://evolution.desktop, dictionnary (sv)
		if (member == 'Update'):
			print('Launcher-API-Daemon: Update',val1,'with',val2)
			
			launcher = None
			if val1 in self.launchers:
				launcher = self.launchers[val1]
				launcher.update_dbus_name(sender)
			else:
				launcher = Launcher(val1, self.dock, sender)
				self.launchers[val1] = launcher
			
			if 'count' in val2:
				count = val2['count']
				launcher.set_count (count)
			
			if 'progress' in val2:
				progress = val2['progress']
				launcher.set_progress (progress)
			
			if 'emblem' in val2:
				emblem = val2['emblem']
				launcher.set_emblem (emblem)
			
			if 'urgent' in val2:
				urgent = val2['urgent']
				launcher.set_urgent (urgent)
			
			if 'quicklist' in val2:
				menu_path = val2['quicklist']
				launcher.set_menu (menu_path)
			
			if 'count-visible' in val2:
				count_visible = val2['count-visible']
				launcher.set_count_visible (count_visible)
			
			if 'progress-visible' in val2:
				progress_visible = val2['progress-visible']
				launcher.set_progress_visible (progress_visible)
			
			if 'emblem-visible' in val2:
				emblem_visible = val2['emblem-visible']
				launcher.set_emblem_visible (emblem_visible)
			
		elif (member == 'Query'):
			print('Launcher-API-Daemon:   query',val2,'for',val1)
		else:
			print('Launcher-API-Daemon: unknown signal')
		
if __name__ == '__main__':
	# check that we don't run twice (ex.: from a 2nd cairo-dock instance)
	# we could detect it from the bus, but then we couldn't detect the following case: Unity launcher + Cairo-Dock
	# we want: ps ux | grep " + filename + " | grep python | grep -v " + pid_str)
	ps_str = popen("ps -u $USER -wwo pid,cmd").read().splitlines() # only users' processes with all lines
	filename = sys.argv[0] # /usr/lib/cairo-dock/cairo-dock-launcher-API-daemon
	pid_str = str (getpid()) # we check if previous processes are running, we exclude this process
	for line in ps_str:
		line_split = line.split()
		line_cmd = ' '.join(line_split[1:])
		line_pid = line_split[0]
		if "python" in line_cmd and filename in line_cmd and not pid_str in line_pid:
			print('Cairo-Dock - Launcher API Daemon is already running (' + line_pid + ')')
			sys.exit(1)
	
	ULWatcher()



#!/usr/bin/env python3
# A simple test case.

from gi.repository import Unity, Gio, GObject, Dbusmenu

loop = GObject.MainLoop()

# Pretend to be Firefox for the sake of the example

launcher = Unity.LauncherEntry.get_for_desktop_id ("firefox.desktop")

# Show a count of 123 on the icon

launcher.set_property("count", 123)

launcher.set_property("count_visible", True)

# Set progress to 40%

launcher.set_property("progress", 0.40)

launcher.set_property("progress_visible", True)


#launcher.set_property("emblem", "play")

#launcher.set_property("emblem_visible", True)


# We also want a quicklist

def on_item_selected(self, menuitem, index):
	print("selected",index)
	
ql = Dbusmenu.Menuitem.new ()

item1 = Dbusmenu.Menuitem.new ()

item1.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item 1")

item1.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)

item1.connect ("item-activated", on_item_selected, 1)

item2 = Dbusmenu.Menuitem.new ()

item2.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item 2")

item2.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)

item2.connect ("item-activated", on_item_selected, 2)

ql.child_append (item1)

ql.child_append (item2)

launcher.set_property("quicklist", ql)

n = 3
prev_item = item2

def update_urgency():
	global prev_item, n
	if launcher.get_property("urgent"):
		print("Removing urgent flag")
		launcher.set_property("urgent", False)
	else:
		print("setting urgent flag")
		launcher.set_property("urgent", True)

	if prev_item == None:
		item = Dbusmenu.Menuitem.new ()
		
		item.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item "+str(n))
		
		item.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
		
		item.connect ("item-activated", on_item_selected, n)
		ql.child_append (item)
		prev_item = item
	else:
		ql.child_delete (prev_item)
		prev_item = None
		item1.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item 1  (%d)"%n)
		n = n+1
	
	return True

GObject.timeout_add_seconds(4, update_urgency)

loop.run()



# This is a part of the external applets for Cairo-Dock
# Copyright : (C) 2010-2011 by Fabounet
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL

# Base class for Cairo-Dock's applets.
# Make your own class derive from a CDApplet, and override the functions you need (the ones which don't start with an underscore).

####################
### dependancies ###
####################
import sys
import os.path

from gi.repository import GLib as glib
from gi.repository import GObject as gobject
g_bMainLoopInGObject = False

import gettext
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import configparser

# Enabling threading support (only for GLib < 2.32)
if glib.glib_version[0] < 2 or (glib.glib_version[0] == 2 and glib.glib_version[1] < 32):
	gobject.threads_init()
else:
	glib.threads_init()

# enable dbus thread support
dbus.mainloop.glib.threads_init()

DBusGMainLoop(set_as_default=True)

INSTALL_PREFIX = os.path.abspath("..")  # applets are launched from their install directory, so ".." is the folder containing all applets.

GETTEXT_NAME = 'cairo-dock-plugins-extra'
LOCALE_DIR = INSTALL_PREFIX + '/locale'  # user version of /usr/share/locale
try:
	translations = gettext.translation(GETTEXT_NAME, LOCALE_DIR)
	try:
		_ = translations.ugettext  # python 2 => force the use of UTF8
	except:
		_ = translations.gettext
except: # e.g. No translation file found for this domain
	_ = lambda x: x

####################
### Applet class ###
####################
class CDApplet:
	
	#############
	### Enums ###
	#############
	# orientation
	BOTTOM = 0
	TOP    = 1
	RIGHT  = 2
	LEFT   = 3
	# container type
	DOCK    = 0
	DESKLET = 1
	# emblem position
	EMBLEM_TOP_LEFT     = 0
	EMBLEM_BOTTOM_RIGHT = 1
	EMBLEM_BOTTOM_LEFT  = 2
	EMBLEM_TOP_RIGHT    = 3
	EMBLEM_MIDDLE       = 4
	EMBLEM_BOTTOM       = 5
	EMBLEM_TOP          = 6
	EMBLEM_RIGHT        = 7
	EMBLEM_LEFT         = 8
	EMBLEM_PERSISTENT   = 0
	EMBLEM_PRINT        = 9
	# menu item types
	MENU_ENTRY        = 0
	MENU_SUB_MENU     = 1
	MENU_SEPARATOR    = 2
	MENU_CHECKBOX     = 3
	MENU_RADIO_BUTTON = 4
	# main menu ID
	MAIN_MENU_ID = 0
	# dialog key pressed
	DIALOG_KEY_ENTER = -1
	DIALOG_KEY_ESCAPE = -2
	
	#####################
	### INIT AND DBUS ###
	#####################
	
	def __init__(self):
		""" initialize the applet. Must be called by any class that inheritates from it.
		It defines the following:
		 - icon : our main icon
		 - sub_icons: our sub-icons
		 - config : a dictionnary where our configuration parameters are stored
		 - cAppletName : name of our applet (the same as the folder where files are stored)
		 - cConfFile : path to our config file (you will rarely need it)
		 """
		self.icon = None
		self.sub_icons = None
		self.config = {}
		self.loop = None
		self._bEnded = False
		self._cMenuIconId = None
		self.cAppletName = sys.argv[0][2:]  # the command is ./applet_name
		self.cBusPath = sys.argv[2]
		self.cConfFile = sys.argv[3]
		self.cRootDataDir = sys.argv[4]
		self.cParentAppName = sys.argv[5]
		self.cShareDataDir = INSTALL_PREFIX + '/' + self.cAppletName
		
		self._get_config()
		self._connect_to_dock()
	
	def _connect_to_dock(self):
		# get our applet on the bus.
		bus = dbus.SessionBus()
		try:
			applet_object = bus.get_object("org.cairodock.CairoDock", self.cBusPath)
		except:
			print(">>> object '"+self.cBusPath+"' can't be found on the bus, exit.\nMake sure that Cairo-Dock is running")
			sys.exit(2)
		self.icon = dbus.Interface(applet_object, "org.cairodock.CairoDock.applet")  # this object represents our icon inside the dock or a desklet.
		sub_icons_object = bus.get_object("org.cairodock.CairoDock", self.cBusPath+"/sub_icons")
		self.sub_icons = dbus.Interface(sub_icons_object, "org.cairodock.CairoDock.subapplet")  # this object represents the list of icons contained in our sub-dock, or in our desklet. We'll add them one by one later, giving them a unique ID, which will be used to identify each of them.
		# connect to signals.
		self.icon.connect_to_signal("on_click", self.on_click)  # when the user left-clicks on our icon.
		self.icon.connect_to_signal("on_middle_click", self.on_middle_click)  # when the user middle-clicks on our icon.
		self.icon.connect_to_signal("on_build_menu", self._on_build_menu)  # when the user right-clicks on our applet (which builds the menu)
		self.icon.connect_to_signal("on_menu_select", self._on_menu_select)  # when the user selects an entry of this menu.
		self.icon.connect_to_signal("on_scroll", self.on_scroll)  # when the user scroll up or down on our icon.
		self.icon.connect_to_signal("on_drop_data", self.on_drop_data)  # when the user drops something on our icon.
		self.icon.connect_to_signal("on_answer_dialog", self.on_answer_dialog)  # when the user answer a question.
		self.icon.connect_to_signal("on_shortkey", self.on_shortkey)  # when the user press the shortkey.
		self.icon.connect_to_signal("on_change_focus", self.on_change_focus)  # when the window's focus changes.
		self.icon.connect_to_signal("on_stop_module", self._on_stop)  # when the user deactivate our applet (or the DBus plug-in, or when the Cairo-Dock is stopped).
		self.icon.connect_to_signal("on_reload_module", self._on_reload)  # when the user changes something in our config, or when the desklet is resized (with no change in the config).
		self.sub_icons.connect_to_signal("on_click_sub_icon", self.on_click_sub_icon)  # when the user left-clicks on a sub-icon.
		self.sub_icons.connect_to_signal("on_middle_click_sub_icon", self.on_middle_click_sub_icon)
		self.sub_icons.connect_to_signal("on_scroll_sub_icon", self.on_scroll_sub_icon)
		self.sub_icons.connect_to_signal("on_build_menu_sub_icon", self._on_build_menu_sub_icon)
		self.sub_icons.connect_to_signal("on_drop_data_sub_icon", self.on_drop_data_sub_icon)
	
	def run(self):
		""" start the applet and enter the main loop; we never get out of this function """
		self.begin()
		if not self._bEnded:  # _bEnded can be true if the applet runs its own main loop, in which case we stay stuck in the 'begin' function until the end of the applet; in this case, we don't want to run a main loop again!
			if g_bMainLoopInGObject:
				self.loop = gobject.MainLoop()
			else:
				self.loop = glib.MainLoop()
			self.loop.run()
		print(">>> applet '"+self.cAppletName+"' terminated.")
		sys.exit(0)
	
	##################################
	### callbacks on the main icon ###
	##################################
	
	def on_click(self,iState):
		""" action on click """
		pass
	
	def on_middle_click(self):
		""" action on middle-click """
		pass
	
	def _on_build_menu(self):
		self._cMenuIconId = None
		self.on_build_menu()
		pass
		
	def on_build_menu(self):
		""" build our menu """
		pass
	
	def _on_menu_select(self,iNumEntry):
		if self._cMenuIconId == None:
			self.on_menu_select(iNumEntry)
		else:
			self.on_menu_select_sub_icon(iNumEntry,self._cMenuIconId)
	
	def on_menu_select(self,iNumEntry):
		""" action on selecting an entry of our menu """
		pass
	
	def on_scroll(self,bScrollUp):
		""" action on scroll """
		pass
	
	def on_drop_data(self,cReceivedData):
		""" action on dropping something on our applet """
	
	def on_answer(self,answer):
		""" action on answering ok to a dialog with ok/cancel buttons (deprecated) """
		pass
	
	def on_answer_dialog(self, button, answer):
		""" action on answering a dialog """
		if button == -1 or button == 0:
			self.on_answer(answer)
	
	def on_shortkey(self,cKey):
		""" action on pressing one of the shortkeys we bound beforehand """
		pass
	
	def on_change_focus(self,bIsActive):
		""" action when the window controlled by the applet takes or looses the focus """
		pass
	
	##################################
	### callbacks on the sub-icons ###
	##################################
	
	def on_click_sub_icon(self, iState, cIconID):
		""" action on click on one of our sub-icons"""
		pass
	
	def on_middle_click_sub_icon(self, cIconID):
		""" action on middle-click on one of our sub-icons"""
		pass
	
	def on_scroll_sub_icon(self, bScrollUp, cIconID):
		""" action on scroll on one of our sub-icons"""
		pass
	
	def _on_build_menu_sub_icon(self, cIconID):
		self._cMenuIconId = cIconID
		self.on_build_menu_sub_icon(cIconID)
	
	def on_build_menu_sub_icon(self, cIconID):
		""" build our menu on one of our sub-icons"""
		pass
	
	def on_menu_select_sub_icon(self, iNumEntry, cIconID):
		""" action on selecting an entry of our menu on a sub-icon"""
		pass
	
	def on_drop_data_sub_icon(self, cReceivedData, cIconID):
		""" action on dropping something on one of our sub-icons"""
		pass
	
	def on_answer_dialog_sub_icon(self, button, answer, cIconID):
		""" action on answering a dialog about a sub-icon"""
		pass
	
	###############################
	### callbacks on the applet ###
	###############################
	
	def begin(self):
		""" action when the applet is started """
		pass
	
	def end(self):
		""" action when the applet is terminated """
		pass
	
	def _on_stop(self):
		self._bEnded = True
		self.end()
		if self.loop != None:
			self.loop.quit()
	
	def reload(self):
		""" called when our applet is reloaded (config parameters have changed) """
		pass
	
	def _on_reload(self,bConfigHasChanged):
		if bConfigHasChanged:
			self._get_config()
			self.reload()
	
	def get_config(self,keyfile):
		""" get our parameters from the key-file """
		pass
	
	def _get_config(self):
		try:
			keyfile = ConfigParser.RawConfigParser() # python 2
		except:
			keyfile = configparser.RawConfigParser() # python 3
		keyfile.read(self.cConfFile)
		self.get_config(keyfile)
	



# This is a part of the Cairo-Dock plug-ins.
# Copyright : (C) 2010-2011 by Fabounet
# E-mail : fabounet@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL

# Base class for Cairo-Dock's main interface.

####################
### dependancies ###
####################
import os.path
import dbus
import re
from dbus.mainloop.glib import DBusGMainLoop

USER_CONFIG_DIR = os.path.expanduser("~/.config")
BUS_NAME = "org.cairodock.CairoDock"

##################
### Main class ###
##################
class CairoDock:
	
	#############
	### Enums ###
	#############
	# orientation
	BOTTOM = 0
	TOP    = 1
	RIGHT  = 2
	LEFT   = 3
	# container type
	DOCK    = "Dock"
	DESKLET = "Desklet"
	# emblem position
	EMBLEM_TOP_LEFT     = 0
	EMBLEM_BOTTOM_RIGHT = 1
	EMBLEM_BOTTOM_LEFT  = 2
	EMBLEM_TOP_RIGHT    = 3
	EMBLEM_MIDDLE       = 4
	EMBLEM_BOTTOM       = 5
	EMBLEM_TOP          = 6
	EMBLEM_RIGHT        = 7
	EMBLEM_LEFT         = 8
	# emblem modifier (optionnal, add it to the emblem position)
	EMBLEM_PERSISTENT   = 0
	EMBLEM_PRINT        = 9
	# module category
	CATEGORY_BEHAVIOR         = 0
	CATEGORY_THEME            = 1
	CATEGORY_APPLET_FILES     = 2
	CATEGORY_APPLET_INTERNET  = 3
	CATEGORY_APPLET_DESKTOP   = 4
	CATEGORY_APPLET_ACCESSORY = 5
	CATEGORY_APPLET_SYSTEM    = 6
	CATEGORY_APPLET_FUN       = 7
	# module type
	CAN_DOCK    = 1
	CAN_DESKLET = 2
	# icon type
	TYPE_LAUNCHER        = "Launcher"
	TYPE_APPLICATION     = "Application"
	TYPE_APPLET          = "Applet"
	TYPE_SEPARATOR       = "Separator"
	TYPE_STACK_ICON      = "Stack-Icon"
	TYPE_CLASS_CONTAINER = "Class-Icon"
	TYPE_OTHER           = "Other"
	# toggle dock visibility
	HIDE_DOCK   = 0
	SHOW_DOCK   = 1
	TOGGLE_DOCK = 2
	
	#####################
	### INIT AND DBUS ###
	#####################
	
	def __init__(self, app_name="cairo-dock"):
		""" Initialize the interface.
		It defines the following:
		 - cDataDir: main dir
		 - cCurrentThemeDir: path to the current theme folder
		 - cConfFile: path to the global config file
		 - cLaunchersDir: path to the launchers folder
		 - cPluginsDir: path to the plug-ins folders
		 - iface: the interface to control the dock
		 """
		
		DBusGMainLoop(set_as_default=True)  # set a main loop for the 'watch_name_owner' signal.
		
		## initialize the class properties
		self.iface = None
		self.cAppName = app_name
		self.cDataDir = USER_CONFIG_DIR + '/' + app_name
		self.cCurrentThemeDir = self.cDataDir + '/current_theme'
		self.cLaunchersDir = self.cCurrentThemeDir + '/launchers'
		self.cPluginsDir = self.cCurrentThemeDir + '/plug-ins'
		self.cConfFile = self.cCurrentThemeDir + '/' + app_name + '.conf'
		
		name1 = self.cAppName.replace('-','')  # -> cairodock
		name2 = re.sub('-[a-z]', lambda x: x.group(0).upper(), self.cAppName)
		name2 = re.sub('^[a-z]', lambda x: x.group(0).upper(), name2)  # -> CairoDock
		name2 = name2.replace('-','')  # -> CairoDock
		self.cObjectPath = '/org/'+name1+'/'+name2
		
		## connect to the dock
		self.bus = dbus.SessionBus()
		self._connect()
		
		## if the watch for de/re-connections
		self.bus.watch_name_owner(BUS_NAME, self.on_name_owner_changed)
	
	def on_name_owner_changed(self,connection_name):
		print("new owner:",connection_name)
		if len(connection_name) == 0:
			if self.iface != None:
				self.iface = None
				print("-> no more connection to "+self.cAppName)
		elif self.iface == None:
			self._connect()
	
	def _connect(self):
		# get gldi on the bus.
		print("connect...")
		try:
			dbus_object = self.bus.get_object(BUS_NAME, self.cObjectPath)
		except:
			print(">>> object '"+self.cObjectPath+"' can't be found on the bus, exit.\nMake sure that Cairo-Dock is running.")
			return
		self.iface = dbus.Interface(dbus_object, "org.cairodock.CairoDock")  # this object represents gldi.
		print("-> connected to "+self.cAppName)
	



# This is a part of the external demo applet for Cairo-Dock
# Copyright : (C) 2011 by matttbe
# E-mail : matttbe@glx-dock.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/licenses/licenses.html#GPL

from setuptools import setup
setup(name='CDBashApplet',
		version='1.0',
		license='GPL-3',
		author='Fabrice Rey',
		author_email='fabounet@glx-dock.org',
		description='Bash interface for Cairo-Dock',
		url='https://launchpad.net/cairo-dock',
		py_modules=['CDBashApplet']
	)
